//
//  Blackjack+ConnectionMgr.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "Blackjack.h"
#import "Blackjack+ConnectionMgr.h"

#define kProcessTimerDelay		0.6     // See note in the processConnectNotifications method for explanation of this.

@implementation Blackjack (ConnectionMgr)

// The three variables below are used by the queueConnectNotification, queueDisconnectNotification, and
// processConnectNotification methods in the Accessory Watchdog Methods section
char connectNotifications = 0;
char disconnectNotifications = 0;
BOOL queueTimerStarted = NO;

#pragma mark - Session Management Methods
// initialize the accessory with the protocolString
- (void)setupForBlackjack:(EAAccessory *)accessory withProtocolString:(NSString *)protocolString
{
    self.accessory = nil;
    self.accessory = accessory;
    self.protocolString = nil;
    self.protocolString =  protocolString;
}

// open a session with the accessory and set up the input and output stream on the default run loop
- (BOOL)openSession
{
    if (self.isConnected) return YES;
    
    [self.accessory setDelegate:self];
    self.session = [[EASession alloc] initWithAccessory:self.accessory forProtocol:self.protocolString];
    
    if (self.session)
    {
        [[self.session inputStream] setDelegate:self];
        [[self.session inputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
        [[self.session inputStream] open];
        
        [[self.session outputStream] setDelegate:self];
        [[self.session outputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
        [[self.session outputStream] open];

        self.isConnected = YES;
    }
    else
    {
        NSLog(@"creating session failed");
    }
    
    return (self.session != nil);
    
}

// close the session with the accessory.
- (void)closeSession
{
	// Closing the streams and releasing session disconnects the app from the XGPS170, but it does not disconnect
	// the XGPS170 from Bluetooth. In other words, the communication streams close, but the device stays
	// registered with the OS as an available accessory.
	//
	// The OS can report that the device has disconnected in two different ways: either that the stream has
	// ended or that the device has disconnected. Either event can happen first, so this method is called
	// in response to a NSStreamEndEventEncountered (from method -stream:handlevent) or in response to an
	// EAAccessoryDidDisconnectNotification (from method -accessoryDisconnected). It seems that the speed of
	// the Apple device being used, e.g. iPod touch gen 2 vs. iPad, affects which event occurs first.
    // Turning off the power on the XGPS170 tends to cause the NSStreamEndEventEncountered to occur
    // before the EAAccessoryDidDisconnectNotification.
	//
    // Note also that a EAAccessoryDidDisconnectNotification is generated when the home button
    // is tapped (bringing up the multitasking menu) beginning in iOS 5.
    
    if (self.session == nil) return;
    
    [[self.session inputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [[self.session inputStream] setDelegate:nil];
    [[self.session inputStream] close];     // call to close moved to here for iOS 7.1 stability
    
    [[self.session outputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [[self.session outputStream] setDelegate:nil];
    [[self.session outputStream] close];    // call to close moved to here for iOS 7.1 stability
    
    self.session = nil;
    self.isConnected = NO;
    
    [self setupForBlackjack:nil withProtocolString:nil];
    
}

- (BOOL)isBlackjackAnAvailableAccessory
{
	BOOL	connect = NO;
    
    if (self.isConnected) return YES;
    
	// get the list of all attached accessories (30-pin or bluetooth)
	NSArray *attachedAccessories = (NSArray *)[[EAAccessoryManager sharedAccessoryManager] connectedAccessories];
    
	for (EAAccessory *obj in attachedAccessories)
	{
		if ([[obj protocolStrings] containsObject:@"com.dualav.xgps170"])
		{
			// At this point, the blackjack has a BT connection to the iPod/iPad/iPhone, but the communication streams
			// have not been opened yet
			connect = YES;
			self.firmwareRev = [NSString stringWithString:[obj firmwareRevision]];
            self.serialNumber = [NSString stringWithString:[obj serialNumber]];
            self.isPaired = YES;
            
            [self setupForBlackjack:obj withProtocolString:@"com.dualav.xgps170"];
            
		}
	}
	
	if (!connect)
	{
		// Blackjack is not available in list of Accessories.");
        self.firmwareRev = NULL;
        self.serialNumber = NULL;
        self.isPaired = NO;
    }
    
	return connect;
}

#pragma mark -
#pragma mark EAAccessoryDelegate methods
- (void)accessoryDidDisconnect:(EAAccessory*)accessory
{
	// don't have to do anything here.
}

#pragma mark -
#pragma mark Accessory watchdog methods
/* When the blackjack connects after being off, the iOS generates a very rapid seqeunce of connect-disconnect-connect
 events. The blackjack device cannot process the resulting stream-open, stream-close, stream-open commands quickly enough
 & may miss the last stream-open command. The solution is to buffer these notifications, and process them after a moment
 of delay.
 
 The queueConnectNotifications method tallies the incoming EAAccessoryDidConnectNotification notifications and kicks
 off a short timer if it hasn't been started already by the queueDisconnectNotification method. The queueDisconnectNotification
 method tallies - you guessed it - the incoming EAAccessoryDidDisconnectNotification notifications.
 
 The processConnectionNotifications method is called when the timer expires. It compares the number of connect and disconnect
 notification and calls the appropriate connect/disconnect method.
 */
- (void)processConnectionNotifications
{
	queueTimerStarted = NO;
    
    if (self.notificationType)   // last notification was to connect
    {
        if ([self isBlackjackAnAvailableAccessory] == YES)
        {
            if ([self openSession] == YES)
            {
                // Notify the view controllers that the blackjack is connected and streaming data
                NSNotification *notification = [NSNotification notificationWithName:@"BlackjackConnected" object:self];
                [[NSNotificationCenter defaultCenter] postNotification:notification];                
            }
        }
    }
    else    // last notification was a disconnect
    {
        // The iOS can send a false disconnect notification when the home button is double-tapped
        // to enter the multitasking menu. So in the event of a EAAccessoryDidDisconnectNotification, double
        // check that the device is actually gone before disconnecting from the blackjack.
        if (self.accessory.connected == YES) return;
        else
        {
            [self closeSession];
            
            // Notify the view controllers that the blackjack disconnected
            NSNotification *notification = [NSNotification notificationWithName:@"BlackjackDisconnected" object:self];
            [[NSNotificationCenter defaultCenter] postNotification:notification];
        }
    }
	
}


- (void)queueDisconnectNotifications:(NSNotification *)notification
{
    self.mostRecentNotification = nil;
    self.mostRecentNotification = notification;
    self.notificationType = NO;
    
    if (queueTimerStarted == NO)
    {
        [self performSelector:@selector(processConnectionNotifications) withObject:nil afterDelay:kProcessTimerDelay];
        queueTimerStarted = YES;
    }

}

- (void)queueConnectNotifications:(NSNotification *)notification
{
    self.mostRecentNotification = nil;
    self.mostRecentNotification = notification;
    self.notificationType = YES;
    
    if (queueTimerStarted == NO)
    {
        [self performSelector:@selector(processConnectionNotifications) withObject:nil afterDelay:kProcessTimerDelay];
        queueTimerStarted = YES;
    }

}

@end
