//
//  Blackjack+LifeCycleMgr.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "Blackjack.h"
#import "Blackjack+LifeCycleMgr.h"
#import "Blackjack+ConnectionMgr.h"
#import "Blackjack+ADSBDataMgr.h"
#import "Blackjack+ControlMgr.h"

#define kMaxNumberOfSatellites  16      // Max number of visible satellites

@implementation Blackjack (LifeCycleMgr)

#pragma mark - Notification Handlers
- (void)startADSBDataFlowing
{
    self.appWantsADSBData = YES;

    if ([self isBlackjackAnAvailableAccessory]) [self openSession];
}

- (void)stopADSBDataFlowing
{
    self.appWantsADSBData = NO;

    [self closeSession];
}

#pragma mark - Application lifecycle
- (void)observeNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(queueConnectNotifications:)
                                                 name:EAAccessoryDidConnectNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(queueDisconnectNotifications:)
                                                 name:EAAccessoryDidDisconnectNotification
                                               object:nil];
    
    // Watch for notifications from the main app turning on/off the ADSB data stream
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(startADSBDataFlowing)
                                                 name:@"ADSBData_Start"
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(stopADSBDataFlowing)
                                                 name:@"ADSBData_Stop"
                                               object:nil];
    
    // Register for notifications from the iOS that accessories are connecting or disconnecting
    [[EAAccessoryManager sharedAccessoryManager] registerForLocalNotifications];
    
}

- (id)init
{
	if ((self = [super init]))
	{
        self.isPaired = NO;
        self.appWantsADSBData = YES;
		self.isConnected = NO;
        self.batteryIsLow = NO;
        self.adsbSessionIsClosed = YES;
        self.sessionsWithBlackjackAreClosed = YES;
		self.firmwareRev = @"";
		self.serialNumber = @"";
        self.latitude = [NSNumber numberWithFloat:0.0]; // must initialize latitude so weather view can scale correctly.
        self.longitude = [NSNumber numberWithFloat:0.0];
        self.altitude = NULL;
        self.velocity = NULL;
        self.heading = NULL;
        self.adsbDataAvailable = 0;
        self.gdl90MessageFragmentContainsValidData = NO;
        self.fisbBlocks = [[NSMutableDictionary alloc] init];
        self.fisbCONUSBlocks = [[NSMutableDictionary alloc] init];
        self.timeOfLastNEXRADReception = NULL;
        self.timeOfLastCONUSNEXRADReception = NULL;
        
        self.detailedGPSInfoEnabled = NO;
        self.basicAndLongTrafficMessagesEnabled = NO;
        self.trafficMessagesEnabled = NO;
        self.messageFilterValueRead = NO;
        
        self.towerList = 0;
        self.messageCount = 0;
        self.towerLocations = [[NSMutableDictionary alloc] initWithCapacity:50];
        
        self.fixType = 0;
        self.gpsPositionIsValid = NO;
        self.waasInUse = NO;
        self.numOfSatInView = [NSNumber numberWithInt:0];
        self.numOfSatInUse = [NSNumber numberWithInt:0];
        self.satsUsedInPosCalc = [[NSMutableArray alloc] init];	// NSNumber integers
		self.dictOfSatInfo = [[NSMutableDictionary alloc] initWithCapacity:kMaxNumberOfSatellites];
        
        // Command queue timer varables
        self.queueTimerStarted = NO;
        self.queue = [[NSMutableArray alloc] init];

        // initialize the GDL90 packet CRC lookup table
        [self crcTableInit];
        
		// Watch for local accessory connect & disconnect notifications. See the note on the queueConnectionNotifications method
        // for more detail.
        [self observeNotifications];
        
        // Check to see if blackjack is attached.
        if ([self isBlackjackAnAvailableAccessory])
        {
            [self openSession];
            [self addCommandRequestToQueue:NSStringFromSelector(@selector(readDeviceSettings))];
        }
    }
	
	return self;
}

- (void)blackjack_applicationWillResignActive
{
    /*
     Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
     Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
     */
	
 	// NOTE: this method is called when:
	//		- when a dialog box (like an alert view) opens. 
	//		- by a double-tap on the home button to bring up the multitasking menu
	//		- when the iPod/iPad/iPhone goes to sleep (manually or after the timer runs out)
	//		- when app exits becuase the home button is tapped (once)
    
	// Close any open streams. The OS sends a false "Accessory Disconnected" message when the home button is double tapped
	// to bring up the mutitasking menu. So the safest thing is to disconnect from the blackjack when that happens, and reconnect
	// later.
	[self closeSession];
	
	// stop watching for Accessory notifications
	[[EAAccessoryManager sharedAccessoryManager] unregisterForLocalNotifications];
}


- (void)blackjack_applicationDidEnterBackground
{
 	// NOTE: this method is called when:
	//		- another app takes forefront. 
	//		- after applicationWillResignActive in response to the home button is tapped (once)
	
	// Close any open streams
	[self closeSession];
	
	// stop watching for Accessory notifications
	[[EAAccessoryManager sharedAccessoryManager] unregisterForLocalNotifications];
}

- (void)blackjack_applicationWillEnterForeground
{
    /*
     Called as part of  transition from the background to the inactive state: here you can undo many of the changes made on entering the background.
     */
	
 	// NOTE: this method is called:
	//		- when an app icon is already running in the background, and the app icon is clicked to resume the app
    
	// Begin watching for Accessory notifications again. Do this first because the rest of the method may complete before
	// the accessory reconnects.
    [self observeNotifications];
	
	// Recheck to see if the blackjack disappeared while away
    if (self.isConnected == NO)
    {
        if ([self isBlackjackAnAvailableAccessory]) [self openSession];
    }
    
}

- (void)blackjack_applicationDidBecomeActive
{
    /*
     Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
     */
    
 	// NOTE: this method is called:
	//		- when an app first opens
	//		- when an app is running & the iPod/iPad/iPhone goes to sleep and is then reawoken, e.g. when the app is
	//		  running->iPod/iPad/iPhone goes to sleep (manually or by the timer)->iPod/iPad/iPhone is woken up & resumes the app
	//		- when the app is resumed from when the multi-tasking menu is opened (in the scenario where the
	//		  app was running, the multitasking menu opened by a double-tap of the home button, followed by a tap on the screen to
	//		  resume the app.)
	
	// begin watching for Accessory notifications again
	////[[EAAccessoryManager sharedAccessoryManager] registerForLocalNotifications];
	[self observeNotifications];
    
	// Recheck to see if the blackjack disappeared while away
    if (self.isConnected == NO)
    {
        if ([self isBlackjackAnAvailableAccessory]) [self openSession];
    }
    
	// NOTE: if the iPod/iPad/iPhone goes to sleep while a view controller is open, there is no notification
	// that the app is back to life, other than this applicationDidBecomeActive method being called. The viewWillAppear,
	// viewDidAppear, or viewDidOpen methods are not triggered when the iPod/iPad/iPhone is woken and the app resumes.
	// Consequently, notify the view controllers in case they need to adjust their UI if the blackjack status changed
    // while the iPod/iPad/iPhone was asleep.
	NSNotification *notification = [NSNotification notificationWithName:@"RefreshUIAfterAwakening" object:self];
	[[NSNotificationCenter defaultCenter] postNotification:notification];
}

- (void)blackjack_applicationWillTerminate
{
    /*
     Called when the application is about to terminate.
     See also applicationDidEnterBackground:.
     */
	
	// Close session with Blackjack
	[self closeSession];
	
	// stop watching for Accessory notifications
	[[EAAccessoryManager sharedAccessoryManager] unregisterForLocalNotifications];
	
	// remove the observers before dealloc is called
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

@end
