//
//  BlackjackDataStructures.h
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define kReserved   1        //0x00000000000001
#define kTRAFFIC    2        //0x00000000000010
#define kNOTAM      4        //0x00000000000100
#define kMETAR      8        //0x00000000001000
#define kTAF        16       //0x00000000010000
#define kDATIS      32       //0x00000000100000
#define kAIRMET     64       //0x00000001000000
#define kSIGMET     128      //0x00000010000000
#define kSUAUPDATE  256      //0x00000100000000
#define kPIREP      512      //0x00001000000000
#define kWINDS      1024     //0x00010000000000
#define kTWIP       2048     //0x00100000000000
#define kRegionalNEXRAD 4096 //0x01000000000000
#define kConusNEXRAD 8192    //0x10000000000000

#ifndef AccConnection_DataStructures_h
#define AccConnection_DataStructures_h

typedef struct
{
    // hour/min/sec in Zulu time
    uint8_t     hour;
    uint8_t     min;
    uint8_t     sec;
    
    // Several different kinds of traffic types. There is one allowed case where a traffic target is
    // either a TIS-B target or a ADS-R target (with an ICAO address in both cases), so both tisbTarget
    // and adsrTarget will be true.
    bool        adsbTarget;
    bool        tisbTarget;
    bool        surfaceVehicle;
    bool        adsbBeacon;
    bool        adsrTarget;
    bool        icaoAddressIsSelfAssigned;
    uint32_t    icaoAddress;
    
    float       latitude;
    float       longitude;
    bool        altitudeType;               // true = pressure altitude, false = geometric altitude
    int32_t     altitude;                   // in feet. values less than -1000 = no altitude data
    float       nic;                        // in meters
    bool        airborne;                   // true = airborne, false = on the ground
    bool        supersonic;                 // true = supersonic, false = subsonic
    int16_t     northerlyVelocity;          // in knots, only valid for airborne vehicles. Positive if N, negative if S
    int16_t     easterlyVelocity;           // in knots, only valid for airborne vehicles. Positive if E, negative if W
    uint16_t    groundSpeed;                // in knots, only valid for surface vehicles
    bool        northerlyVelocityAvail;     // false = no valid velocity reported. Velocity value should be considered invalid
    bool        easterlyVelocityAvail;      // false = no valid velocity reported. Velocity value should be considered invalid
    bool        groundSpeedAvail;           // false = no valid groundspeed reported. Speed value should be considered invalid
    
    // if trueTrackAngle, magenticHeading and trueHeading are all false, the heading data not available and heading value will be -1
    bool        trueTrackAngle;             // true if heading value is a track angle
    bool        magneticHeading;            // true if heading value is magnetic
    bool        trueHeading;                // true if heading value is true
    float       heading;                    // in degrees. If heading information not available, value = -1
    
    // for airborne vehicles, there is velocity information. if not airborne, there is vehicle size information
    bool        verticalVelocitySource;     // true = barometric source, false = geometric source
    bool        verticalVelocityDirection;  // true = down, false = up
    int16_t     verticalVelocityRate;       // in feet per minute
    
    // for ground-based objects, size & GPS antenna offset information can be available.
    bool        vehicleSizeIsValid;
    uint8_t     length;                     // length of aircraft/vehicle in meters
    float       width;                      // width of aircraft/vehicle in meters
    bool        gpsAntennaInfoIsValid;
    bool        gpsAntennaOffsetAxis;               // true = longitudinal antenna offset, false = lateral antenna offset
    bool        gpsAntennaOffsetSide;               // valid for lateral offset only: true = right, false = left
    uint8_t     gpsAntennaOffsetDistance;           // in meters, from the longitudinal (roll) axis of the aircraft or from the nose of the aircraft
    
    // depending on the type of transmitter source
    bool        adsbTransmittingSourceCoupledToUTCTime;
    uint8_t     uplinkFeedback;             // max number of successful uplink messages received
    uint8_t     tisbSiteID;
    
    bool        isBasicMessage;             // true if it's a Basic traffic message, false if Long
    // Basic traffic messages only have the information above. Long traffic reports have these additional items:
    
    // Emmitter category is the description of the aircraft, e.g. large vortex heavy, paraglider, UAV, etc.
    char        emitterCategoryDescr[38];
    
    // The tail number is sometimes available
    char        callSignFlightPlanID[9];
    
    // The type of emergency is sometimes available
    char        statusDescr[24];
    
    // Other
    uint8_t     mopsVersion;
    uint8_t     sil;
    uint8_t     transmitMSO;
    uint8_t     sda;
    uint8_t     nacP;
    uint8_t     nacV;
    BOOL        nicBaro;
    uint8_t     capabilityCodes;
    uint8_t     operationalModes;
    BOOL        csid;
    BOOL        silSUPP;
    uint8_t     geoVertAcc;
    BOOL        saFlag;
    BOOL        nicSUPP;
    
} LongTrafficObjectInfo;

typedef struct
{
    unsigned char   status;
    unsigned char   addressType;
    uint32_t        icaoAddress;
    float           latitude;
    float           longitude;
    int32_t         altitude;
    unsigned char   miscIndicators;
    unsigned char   nic;
    unsigned char   nacP;
    uint16_t        horizVelocity;
    int16_t         vertVelocity;
    float           trackHeading;
    uint8_t         emitterCategory;
    char            callsign[8];
    unsigned char   emergencyPriorityCode;
} ShortTrafficObjectInfo;

#endif