//
//  UIView+CommonDrawingFuncs.m
//  XGPS170 Developers Kit.
//
//  Version 2.5.1
//  Licensed under the terms of the BSD License, as specified below.

/*
 Copyright (c) 2015 Dual Electronics Corp.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must strong the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the name of the Dual Electronics Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "UIView+CommonDrawingFuncs.h"
#import <CoreText/CoreText.h>

#define kTickSize   3.0

@implementation UIView (CommonDrawingFuncs)

static inline double radians (double degrees) { return degrees * M_PI/180; }

void drawLinearGradient(CGContextRef context, CGRect rect, CGColorRef startColor, CGColorRef  endColor) {
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGFloat locations[] = { 0.0, 1.0 };
    
    NSArray *colors = [NSArray arrayWithObjects:(__bridge id)startColor, (__bridge id)endColor, nil];
    
    CGGradientRef gradient = CGGradientCreateWithColors(colorSpace, (CFArrayRef) colors, locations);
    
    CGPoint startPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMinY(rect));
    CGPoint endPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMaxY(rect));
    
    CGContextSaveGState(context);
    CGContextAddRect(context, rect);
    CGContextClip(context);
    CGContextDrawLinearGradient(context, gradient, startPoint, endPoint, 0);
    CGContextRestoreGState(context);
    
    CGGradientRelease(gradient);
}

void drawGlossAndGradient(CGContextRef context, CGRect rect, CGColorRef startColor, CGColorRef endColor) {
    
    drawLinearGradient(context, rect, startColor, endColor);
    
    CGColorRef glossColor1 = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.35].CGColor;
    CGColorRef glossColor2 = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.1].CGColor;
    
    CGRect topHalf = CGRectMake(rect.origin.x, rect.origin.y, rect.size.width, rect.size.height/2);
    
    drawLinearGradient(context, topHalf, glossColor1, glossColor2);
    
}

CGMutablePathRef createRoundedRectForRect(CGRect rect, CGFloat radius) {
    
    CGMutablePathRef path = CGPathCreateMutable();
    CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
    CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect),
                        CGRectGetMaxX(rect), CGRectGetMaxY(rect), radius);
    CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect),
                        CGRectGetMinX(rect), CGRectGetMaxY(rect), radius);
    CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect),
                        CGRectGetMinX(rect), CGRectGetMinY(rect), radius);
    CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect),
                        CGRectGetMaxX(rect), CGRectGetMinY(rect), radius);
    CGPathCloseSubpath(path);
    
    return path;
}

- (void)drawGradientCircle:(CGContextRef)context
              centerPointX:(float)centerX
              centerPointY:(float)centerY
                    radius:(float)radius
              initialColor:(CGColorRef)topColor
                 fadeColor:(CGColorRef)bottomColor
{
    CGRect outerRect = CGRectMake(centerX - radius, centerY - radius, 2.0*radius, 2.0*radius);
    CGMutablePathRef outerPath = createRoundedRectForRect(outerRect, radius);
    
    CGContextSaveGState(context);
    CGContextAddPath(context, outerPath);
    CGContextClip(context);
    drawGlossAndGradient(context, outerRect, topColor, bottomColor);
    CGContextRestoreGState(context);
}

// draw a circle centered on the passed in x,y coordinates
- (void) drawCenteredCircle:(CGContextRef)context
			   centerPointX:(float)centerX
			   centerPointY:(float)centerY
					 radius:(int)radius
			 colorForStroke:(UIColor *)strokeColor
			   colorForFill:(UIColor *)fillColor
{
	float sides = 100;
	
	CGContextSaveGState(context);
	
	if (strokeColor == NULL) CGContextSetStrokeColorWithColor(context, [UIColor clearColor].CGColor);
	else CGContextSetStrokeColorWithColor(context, strokeColor.CGColor);
	
	if (fillColor == NULL) CGContextSetFillColorWithColor(context, [UIColor clearColor].CGColor);
	else CGContextSetFillColorWithColor(context, fillColor.CGColor);
	
	CGContextSetLineWidth(context, 2);
	
	for(int i = 0; i <= sides; i++)
	{
		CGFloat pointRatio = (float)i / sides;
		CGFloat xSteps = cos( pointRatio * 2.0f * M_PI);
		CGFloat ySteps = sin( pointRatio * 2.0f * M_PI );
		CGFloat pointX = centerX + xSteps * (float)radius;
		CGFloat pointY = centerY + ySteps * (float)radius;
		//NSLog(@"%i", pointX);
		
		if (i == 0) CGContextMoveToPoint(context, pointX, pointY);
		CGContextAddLineToPoint(context, pointX, pointY);
	}
	CGContextDrawPath(context, kCGPathFillStroke);
    
	CGContextRestoreGState(context);
}

// draw crosshairs centered on the passed in x,y coordinates
- (void) drawCrosshairs:(CGContextRef)context
           centerPointX:(float)centerX
           centerPointY:(float)centerY
				   size:(int)size
		 colorForStroke:(UIColor *)strokeColor
{
	CGContextSaveGState(context);
	
	CGContextSetLineCap(context, kCGLineCapSquare);
	CGContextSetStrokeColorWithColor(context, strokeColor.CGColor);
	CGContextSetLineWidth(context, 2.0);
	// draw horizontal bar
	CGContextMoveToPoint(context, centerX - ((float)size/2.0f), centerY);
	CGContextAddLineToPoint(context, centerX + ((float)size/2.0f), centerY);
	CGContextStrokePath(context);
	// draw vertical bar
	CGContextMoveToPoint(context, centerX, centerY - ((float)size/2.0f));
	CGContextAddLineToPoint(context, centerX, centerY + ((float)size/2.0f));
	CGContextStrokePath(context);
    
	CGContextRestoreGState(context);
} // drawCrosshairs

// draw crosshairs centered on the passed in x,y coordinates
- (void) drawRadarAxes:(CGContextRef)context
          centerPointX:(float)centerX
          centerPointY:(float)centerY
           radarRadius:(float)radarRadius
             tickScale:(float)tickScale
      reticleLineWidth:(float)lineWidth
        colorForStroke:(UIColor *)strokeColor
{
	CGContextSaveGState(context);
	
	CGContextSetLineCap(context, kCGLineCapSquare);
	CGContextSetStrokeColorWithColor(context, strokeColor.CGColor);
	CGContextSetLineWidth(context, lineWidth);
    
	// draw horizontal bar
	CGContextMoveToPoint(context, 0, centerY);
	CGContextAddLineToPoint(context, self.bounds.size.width, centerY);
	CGContextStrokePath(context);
    
	// draw vertical bar
	CGContextMoveToPoint(context, centerX, 0);
	CGContextAddLineToPoint(context, centerX, self.bounds.size.height);
	CGContextStrokePath(context);
    
    // draw tick marks...
    float tickSpacing = self.bounds.size.height / 2.0 / radarRadius * tickScale;
    
    // ...along the horizontal axis
    for (int i=0; i<self.bounds.size.width; i++)
    {
        CGContextMoveToPoint(context, i*tickSpacing, centerY - kTickSize);
        CGContextAddLineToPoint(context, i*tickSpacing, centerY + kTickSize);
        CGContextStrokePath(context);
        
        CGContextMoveToPoint(context, centerX - kTickSize, i*tickSpacing);
        CGContextAddLineToPoint(context, centerX + kTickSize, i*tickSpacing);
        CGContextStrokePath(context);
    }
    
	CGContextRestoreGState(context);
} // drawCrosshairs

// draw frame around image
- (void) drawBoundingBox:(CGContextRef)context
				   inset:(float)insetDist
		  colorForStroke:(UIColor *)strokeColor
		  axesStrokeWidth:(float)axesStrokeWidth
{
	CGContextSaveGState(context);
	
	CGContextSetLineCap(context, kCGLineCapSquare);
	CGContextSetStrokeColorWithColor(context, strokeColor.CGColor);
	CGContextSetLineWidth(context, axesStrokeWidth);
    
    CGRect rectangle = CGRectMake(self.bounds.origin.x + insetDist,
                                  self.bounds.origin.y + insetDist,
                                  self.bounds.size.width - (2.0 * insetDist),
                                  self.bounds.size.height - (2.0 * insetDist));
    
    CGContextAddRect(context, rectangle);
    
    CGContextStrokePath(context);
	
	CGContextRestoreGState(context);
} // drawBoundingBox

- (void)drawLegendText:(CGContextRef)context text:(NSString *)displayText
{
    // Flip the coordinate system
    CGContextSetTextMatrix(context, CGAffineTransformIdentity);
    CGContextTranslateCTM(context, 0, self.bounds.size.height);
    CGContextScaleCTM(context, 1.0, -1.0);
    
    CGMutablePathRef path = CGPathCreateMutable();
    CGPathAddRect(path, NULL, self.bounds );
    
    /*
     CTFontRef fontRef = CTFontCreateWithName((CFStringRef)self.font,
     24.0f, NULL);
     NSDictionary* attrs = [NSDictionary dictionaryWithObjectsAndKeys:
     (id)self.color.CGColor, kCTForegroundColorAttributeName,
     (id)fontRef, kCTFontAttributeName,
     (id)self.strokeColor.CGColor, (NSString *) kCTStrokeColorAttributeName,
     (id)[NSNumber numberWithFloat: self.strokeWidth], (NSString *)kCTStrokeWidthAttributeName,
     nil];
     */
    
    NSDictionary* stringAttrs =
    [NSDictionary dictionaryWithObjectsAndKeys:
     (id)[UIColor whiteColor].CGColor, kCTForegroundColorAttributeName,
     (id)[UIColor whiteColor].CGColor, (NSString *)kCTStrokeColorAttributeName, nil];
    
    NSAttributedString* attString = [[NSAttributedString alloc]
                                      initWithString:displayText attributes:stringAttrs];
    
    CTFramesetterRef framesetter = CTFramesetterCreateWithAttributedString((CFAttributedStringRef)attString);
    CTFrameRef frame = CTFramesetterCreateFrame(framesetter, CFRangeMake(0, [attString length]), path, NULL);
    
    CTFrameDraw(frame, context);
    
    CFRelease(frame);
    CFRelease(path);
    CFRelease(framesetter);
}

// draw a hemicircle reticle
- (void) drawHemiCircle:(CGContextRef)context
           centerPointX:(float)centerX
           centerPointY:(float)centerY
                 radius:(int)radius
         colorForStroke:(UIColor *)strokeColor
           colorForFill:(UIColor *)fillColor
{
	float sides = 100;
	
	CGContextSaveGState(context);
	
	if (strokeColor == NULL) CGContextSetStrokeColorWithColor(context, [UIColor clearColor].CGColor);
	else CGContextSetStrokeColorWithColor(context, strokeColor.CGColor);
	
	if (fillColor == NULL) CGContextSetFillColorWithColor(context, [UIColor clearColor].CGColor);
	else CGContextSetFillColorWithColor(context, fillColor.CGColor);
	
	CGContextSetLineWidth(context, 2);
	
	for(int i = 0; i <= sides; i++)
	{
		CGFloat pointRatio = (float)i / sides;
		CGFloat xSteps = cos( (pointRatio * M_PI) + M_PI);
		CGFloat ySteps = sin( (pointRatio * M_PI) + M_PI);
		CGFloat pointX = centerX + xSteps * (float)radius;
		CGFloat pointY = centerY + ySteps * (float)radius;
		if (i == 0) CGContextMoveToPoint(context, pointX, pointY);
		CGContextAddLineToPoint(context, pointX, pointY);
	}
	CGContextDrawPath(context, kCGPathFillStroke);
    
    // add tickmarks every 10 degrees
    float ax, ay, bx, by, tickLen, minorTickLen, majorTickLen;
    minorTickLen = 6;
    majorTickLen = 10;
    float angle = 0;
    unsigned char c = 0;
    bool showMajorTick = NO;
    for (int j=0; j<19; j++)
    {
        if (showMajorTick) tickLen = majorTickLen;
        else tickLen = minorTickLen;
        
        ax = ((radius - (0.5 * tickLen)) * cos (angle + M_PI)) + centerX;
        ay = ((radius - (0.5 * tickLen)) * sin (angle + M_PI)) + centerY;
        bx = ((radius + (0.5 * tickLen)) * cos (angle + M_PI)) + centerX;
        by = ((radius + (0.5 * tickLen)) * sin (angle + M_PI)) + centerY;
        
        CGContextMoveToPoint(context, ax, ay);
        CGContextAddLineToPoint(context, bx, by);
        CGContextDrawPath(context, kCGPathFillStroke);
        
        angle += radians(180/18);
        c++;
        if (c == 3)
        {
            showMajorTick = YES;
            c = 0;
        }
        else showMajorTick = NO;
    }
    CGContextRestoreGState(context);
}

// draw a circle centered on the passed in x,y coordinates
- (void) drawCompassRose:(CGContextRef)context
            centerPointX:(float)centerX
            centerPointY:(float)centerY
                  radius:(float)radius
                   scale:(float)radiusScaleFactor
         angleOfRotation:(float)rotationAngle
                fontSize:(float)fontSize
         axesStrokeWidth:(float)axesStrokeWidth
          colorForStroke:(UIColor *)strokeColor
            colorForFill:(UIColor *)fillColor
{
	float sides = 100;
    radius *= radiusScaleFactor;
	
	CGContextSaveGState(context);
	
	if (strokeColor == NULL) CGContextSetStrokeColorWithColor(context, [UIColor clearColor].CGColor);
	else CGContextSetStrokeColorWithColor(context, strokeColor.CGColor);
	
	if (fillColor == NULL) CGContextSetFillColorWithColor(context, [UIColor clearColor].CGColor);
	else CGContextSetFillColorWithColor(context, fillColor.CGColor);
	
	CGContextSetLineWidth(context, axesStrokeWidth);
	
	for(int i = 0; i <= sides; i++)
	{
		CGFloat pointRatio = (float)i / sides;
		CGFloat xSteps = cos( pointRatio * 2.0f * M_PI);
		CGFloat ySteps = sin( pointRatio * 2.0f * M_PI );
		CGFloat pointX = centerX + xSteps * (float)radius;
		CGFloat pointY = centerY + ySteps * (float)radius;
		//NSLog(@"x,y = %.1f,%.1f", pointX, pointY);
		
		if (i == 0) CGContextMoveToPoint(context, pointX, pointY);
		CGContextAddLineToPoint(context, pointX, pointY);
	}
	CGContextDrawPath(context, kCGPathFillStroke);
    
    // add tickmarks every 10 degrees
    CGFloat ax, ay, bx, by, tickLen, minorTickLen, majorTickLen;
    minorTickLen = 6.0;
    majorTickLen = 10.0;
    float angle = radians(rotationAngle - 90);
    unsigned char c = 0;
    bool showMajorTick = NO;
    NSString *angleText = nil;
    
    for (int j=36; j>-1; j--)
    {
        if (showMajorTick) tickLen = majorTickLen;
        else tickLen = minorTickLen;
        
        ax = ((radius - (0.5 * tickLen)) * cosf((-1.0 * angle) + M_PI)) + centerX;
        ay = ((radius - (0.5 * tickLen)) * sinf((-1.0 * angle) + M_PI)) + centerY;
        bx = ((radius + (0.5 * tickLen)) * cosf((-1.0 * angle) + M_PI)) + centerX;
        by = ((radius + (0.5 * tickLen)) * sinf((-1.0 * angle) + M_PI)) + centerY;
        
        CGContextMoveToPoint(context, ax, ay);
        CGContextAddLineToPoint(context, bx, by);
        CGContextDrawPath(context, kCGPathFillStroke);
        
        if (showMajorTick)
        {
            CGContextSaveGState(context);
            angleText = [NSString stringWithFormat:@"%d", j];
            CGContextSetTextDrawingMode (context, kCGTextFill);
            //CGContextSelectFont (context, "Helvetica", fontSize, kCGEncodingMacRoman);
            CGContextSetRGBFillColor (context, 0.75, 0.75, 0.75, 1);
            CGContextTranslateCTM(context, 0, 2*centerY);
            CGContextScaleCTM(context, 1, -1);
            
            float textOffsetX, textOffsetY;
            float textWidth = 12.0;
            float textHeight = 6.0;
            
            if (j < 10) textWidth /= 2.0;
            
            if (angle > M_PI) textOffsetX = (((2.0 * M_PI) - angle) / M_PI) * textWidth;
            else if (angle < (-1.0* M_PI)) textOffsetX = (((2.0 * M_PI) + angle) / M_PI) * textWidth;
            else textOffsetX = fabs(angle / M_PI) * textWidth;
            
            if (angle > (3.0 * M_PI / 2.0)) textOffsetY = ((M_PI_2 - angle) + (2.0 * M_PI)) / M_PI * textHeight;
            else if (angle < (-1.0 * M_PI_2)) textOffsetY = (angle + (3.0 * M_PI / 2.0)) / M_PI * textHeight;
            else textOffsetY = fabs(M_PI_2 - angle) / M_PI * textHeight;
            //NSLog(@"j = %d. angle = %.2f. textOffsetX,textOffsetY = %.1f,%.1f", j, angle, textOffsetX, textOffsetY);
            
            float labelInset = 10.0;
            float x = ((radius-labelInset) * cosf(angle + M_PI)) - textOffsetX + centerX;
            float y = ((radius-labelInset) * sinf(angle + M_PI)) - textOffsetY + centerY;
            //CGContextShowTextAtPoint (context, x, y, [angleText UTF8String],[angleText lengthOfBytesUsingEncoding:NSASCIIStringEncoding]);
            [angleText drawAtPoint:CGPointMake(x,y)
                              withAttributes:@{NSFontAttributeName:[UIFont fontWithName:@"Helvetica"
                                                                                   size:fontSize]
                                               }];

            CGContextRestoreGState(context);
            
        }
        
        angle += radians(180.0f/18.0f);
        if (angle > (2.0 * M_PI)) angle -= (2.0 * M_PI);
        c++;
        if (c == 3)
        {
            showMajorTick = YES;
            c = 0;
        }
        else showMajorTick = NO;
    }
    
	CGContextRestoreGState(context);
}

// Draw the grid that shows when the image is panned off center.
- (void)drawReticleGrid:(CGContextRef)context
           gridSizeInNM:(unsigned int)dist
        axesStrokeWidth:(float)axesStrokeWidth
         colorForStroke:(UIColor *)strokeColor
            scaleFactor:(float)scaleFactor

{
    CGContextSaveGState(context);
	
	CGContextSetLineCap(context, kCGLineCapSquare);
	CGContextSetStrokeColorWithColor(context, strokeColor.CGColor);
	CGContextSetLineWidth(context, axesStrokeWidth);
    
    // bar length will rotate as the plane rotates, so the bars must be long enough to reach from corner to corner
    float barLength = sqrtf((pow(self.frame.size.width,2) + pow(self.frame.size.height,2)));
    
	// draw vertical bars
    float widthSpacing = (float)dist * scaleFactor;
    for (float i=0.0; i<self.frame.size.width; i+=widthSpacing)
    {
        CGContextMoveToPoint(context, (widthSpacing / 2.0) - i, -0.5 * (barLength - self.frame.size.width));
        CGContextAddLineToPoint(context, (widthSpacing / 2.0) - i, self.frame.size.width + (0.5 * (barLength - self.frame.size.width)));
        CGContextStrokePath(context);
        
        CGContextMoveToPoint(context, (widthSpacing / 2.0) + i, -0.5 * (barLength - self.frame.size.width));
        CGContextAddLineToPoint(context, (widthSpacing / 2.0) + i, self.frame.size.width + (0.5 * (barLength - self.frame.size.width)));
        CGContextStrokePath(context);
    }
    
	// draw horizontal bars
    float heightSpacing = (float)dist * scaleFactor;
    for (float i=0.0; i<self.frame.size.height; i+=heightSpacing)
    {
        CGContextMoveToPoint(context, 0, (heightSpacing / 2.0) - i);
        CGContextAddLineToPoint(context, barLength, (heightSpacing / 2.0) - i);
        CGContextStrokePath(context);
        
        CGContextMoveToPoint(context, 0, (heightSpacing / 2.0) + i);
        CGContextAddLineToPoint(context, barLength, (heightSpacing / 2.0) + i);
        CGContextStrokePath(context);
    }
    
	CGContextRestoreGState(context);
} // drawReticleGrid

@end
